//*****************************************************************************
// FILE:  ASSNMANY.H
//
// DESCRIPTION:
//
//      Contains all classes needed to implement one-to-many and
//      many-to-many binary associations.
//
// REVISION HISTORY:
//  DATE      WHO   DESCRIPTION
//  11/04/95  TJL   Created.
// 
//*****************************************************************************

#define AS_ASSNMANY_H

#ifndef AS_ASSN_H
#include "assn.h"
#endif 

#ifndef AS_ASSNLIST_H
#include "assnlist.h"
#endif

//*****************************************************************************
//
// Class ASToManyBase
//
// This association base class is used by objects that are associated with
// many objects. 
//
//*****************************************************************************

template <class lhsClass, 
          class rhsClass, 
          class propClass,
          class listItemClass
         >
class ASToManyBase : public ASAssn< lhsClass, rhsClass, propClass >
{
  protected:

    typedef ASIteratorAdapter<listItemClass> iteratorAdapterClass;

    typedef ASListAdapter<listItemClass,
                          rhsAssnClass
                         > listAdapterClass;

  public:

    ASToManyBase( lhsClass & rItem, listAdapterClass &rList );
    ~ASToManyBase();
    
    int IsAttached() const;
    const listAdapterClass &GetRhsObjects() const;
    
  protected:
  
    listAdapterClass &m_rAssnList;
};

//*****************************************************************************
//
// Class ASToMany
//
// This association class is used by objects that are associated with
// many objects.  No properties.   Ordered or unordered (depends on how 
// Attach() is called).
//
//*****************************************************************************

template <class lhsClass, 
          class rhsClass 
         >
class ASToMany
  : public ASToManyBase< lhsClass,  
                         rhsClass, 
                         ASNoProperties,
                         ASAssn<rhsClass, lhsClass, ASNoProperties>
                       >
{
  private:

    typedef ASToMany< lhsClass, rhsClass > selfClass;

  public:

    ASToMany( lhsClass &rItem, listAdapterClass &rList );
    ~ASToMany();

    void Attach( rhsAssnClass & rAssociationToLinkTo,
                 int iOrder = 0    
               );
               
    void Detach( rhsAssnClass &rAssociation );
    void DetachMe();
    
  private:
  
    ASToMany( selfClass& );  // No implementation
    operator = ( selfClass& ); // No implementation
    operator = ( lhsClass & ); // No implementation
     
    void iAttach( rhsAssnClass &rAssociation,
                  ASNoProperties * const = NULL,
                  const int iOrder = 0
                );

    void iDetach( const rhsAssnClass &rAssociation );
};

//*****************************************************************************
//
// Class ASToManyProp
//
// This association class is used by objects that are associated with
// many objects.  Association with properties.  Ordered or unordered
// (depends on how Attach() is called).
//
//*****************************************************************************

#ifndef AS_ASTUPLE_H
#include "astuple.h"
#endif

template <class lhsClass, 
          class rhsClass, 
          class propClass
         >
class ASToManyProp
  : public ASToManyBase< lhsClass,  
                         rhsClass, 
                         propClass, 
                         ASAssnPropTuple< rhsClass,  lhsClass,  propClass >
                       >
{
  private:

    typedef ASToManyProp<lhsClass, rhsClass,
      propClass> selfClass;

    typedef ASAssnPropTuple<rhsClass, lhsClass,  
      propClass> listItemClass;

  public:

    ASToManyProp( lhsClass & rItem, 
                  listAdapterClass & rList, 
                  const ASDeletePropertiesEnum eDeleteProperties 
                    = ASDP_DELETE_PROPERTIES 
                );
    ~ASToManyProp();
    
    void Attach( rhsAssnClass & rAssociationToLinkTo,
                 propClass &rProperties,
                 int iOrder = 0
               );
    
    void Detach( rhsAssnClass &rAssociation );
    void DetachMe();
    
  private:
  
    const ASDeletePropertiesEnum m_eDeleteProperties;

    ASToManyProp( selfClass& ); // No implementation
    operator = ( selfClass&  ); // No implementation
    operator = ( lhsClass & ); // No implementation

    void iAttach( rhsAssnClass &rAssociation,
                  propClass * const pProperties,
                  const int iOrder = 0
                );
    void iDetach( const rhsAssnClass &rAssociation );
};
