//*****************************************************************************
// FILE:  ASSN_ONE.H
//
// DESCRIPTION:
//
//      Contains all classes needed to implement one-to-one binary 
//      associations.
//
// REVISION HISTORY:
//  DATE      WHO   DESCRIPTION
//  11/04/95  TJL   Created.
// 
//*****************************************************************************

#define AS_ASSN_ONE_H

#ifndef AS_ASSN_H
#include "assn.h"
#endif 

//*****************************************************************************
//
// Class ASToOneBase
//
// This association base class is used by objects that are associated with
// only one object. 
//
//*****************************************************************************

template <class lhsClass, class rhsClass, class propClass>
class ASToOneBase : public ASAssn< lhsClass, 
                                   rhsClass, 
                                   propClass
                                 >
{
  public:

    ASToOneBase( lhsClass & rItem );
    
    int IsAttached() const;
    rhsClass &GetRhsObject();
    
  protected:
  
    rhsAssnClass *m_pRhsAssociation;
};

//*****************************************************************************
//
// Class ASToOne
//
// This association class is used by objects that are associated with
// only one object. The association has no properties.
//
//*****************************************************************************

template <class lhsClass, class rhsClass>
class ASToOne : public
  ASToOneBase< lhsClass, rhsClass, ASNoProperties >
{
  private:

   typedef ASToOne<lhsClass, rhsClass> selfClass;
   
  public:

    ASToOne( lhsClass & rItem );
    ~ASToOne();
    
    void Attach( rhsAssnClass & rAssociationToLinkTo );
    void DetachMe();
    
  private:
  
    ASToOne( selfClass& ); // No implementation
    operator = ( selfClass& ); // No implementation
    operator = ( lhsClass & ); // No implementation

    void iAttach( rhsAssnClass & rAssociation,
                  ASNoProperties * const = NULL,
                  const int iOrder = 0
                );
                
    void iDetach( const rhsAssnClass &rAssociation );
};

//*****************************************************************************
//
// Class ASToOneProp
//
// This association class is used by objects that are associated with
// only one object. 
//
//*****************************************************************************

template <class lhsClass, class rhsClass, class propClass >
class ASToOneProp  
  : public ASToOneBase< lhsClass, rhsClass, propClass >
{
  private:

    typedef ASToOneProp< lhsClass, rhsClass, propClass > selfClass;

  public:

    ASToOneProp( lhsClass & rItem, 
                 const ASDeletePropertiesEnum eDeleteProperties =  
                  ASDP_DELETE_PROPERTIES
               );
    ~ASToOneProp();
    
    void Attach( rhsAssnClass & rAssociationToLinkTo,
                 propClass & rProperties
               );
    propClass& GetProperties();
    void DetachMe();
   
  private:
  
    propClass *m_pProperties;
    const ASDeletePropertiesEnum m_eDeleteProperties;
    
    ASToOneProp( selfClass& ); // No implementation
    operator = ( selfClass& ); // No implementation
    operator = ( lhsClass & ); // No implementation

    void iAttach( rhsAssnClass & rAssociation,
                  propClass * const pProperties = NULL,
                  const int iOrder = 0
                );
    
    void iDetach( const rhsAssnClass & );
};
