//*****************************************************************************
//
// FILE:  ASTUPLE.H
//
// DESCRIPTION:
//
//      Contains class definition for ASAssnPropTuple.
//
// REVISION HISTORY:
//  DATE      WHO   DESCRIPTION
//  11/03/95  TJL   Created.
// 
//*****************************************************************************

#define AS_ASTUPLE_H

#ifndef AS_ASSN_H
#include "assn.h"
#endif

//*****************************************************************************
//
// class ASAssnPropTuple
//
// Contains an (association, property) pair.
//
// Objects of this class are stored in a list for one-to-many relationships
// that have properties.
//
//*****************************************************************************

template <class lhsClass, class rhsClass, class propClass>
class ASAssnPropTuple
{
  private:

    typedef ASAssnPropTuple< lhsClass, rhsClass, propClass> selfClass;

  public:
    
    // public methods
    
    ASAssnPropTuple( ASAssn<lhsClass, 
                            rhsClass, 
                            propClass
                           > &rAssociation,
                     propClass &rProperties,
                     const ASDeletePropertiesEnum eDeleteProperties 
                   );
    ~ASAssnPropTuple();
    
    int operator == (const ASAssn<lhsClass, 
                                         rhsClass, 
                                         propClass> &rAssociation
                    ) const;
    ASAssn<lhsClass, rhsClass, propClass>& GetAssn();
    propClass& GetProperties();
    void DoNotDeleteProperties();
    
  private:
  
    ASDeletePropertiesEnum m_eDeleteProperties;
    ASAssn<lhsClass, rhsClass, propClass> &m_rAssociation;
    propClass &m_rProperties;

    selfClass( selfClass& );  // No implementation
    operator = (selfClass&);  // No implementation
};
