//*****************************************************************************
// FILE:  CAR.CPP
//
// DESCRIPTION:
//
//      Test program for associations.  Car / installed components example.
//
// REVISION HISTORY:
//  DATE      WHO   DESCRIPTION
//  1l/04/95  TJL   Created.
// 
//*****************************************************************************

#include <stdio.h>

#include "assn.h"
#include "list.h"

// Forward
class Car;
class Accessory;
class CarAccessoryProperties;

//*****************************************************************************
//
// Typedefs for list and iterator 
//
//*****************************************************************************

typedef AS_LIST_PROP( ListTempl, Car, Accessory, CarAccessoryProperties ) List;
typedef AS_ITER_ADAPTER_PROP( Car, Accessory, CarAccessoryProperties ) Iterator;

//*****************************************************************************
//
// class Car 
//
//*****************************************************************************

class Car
{
  public:
  
  Car() : m_hasAccessories( *this, *new List )
  {
  }
  
  ASToManyProp< Car, Accessory, CarAccessoryProperties > m_hasAccessories;
};

//*****************************************************************************
//
// class Accessory 
//
//*****************************************************************************

class Accessory
{
  public:
  
  Accessory() : m_installedIn( *this )
  {}
  
  ASToOneProp<Accessory, Car, CarAccessoryProperties> m_installedIn;
};

//*****************************************************************************
//
// class CarAccessoryProperties
//
//*****************************************************************************

class CarAccessoryProperties
{
  public:
  
  CarAccessoryProperties( long cost )
    : m_installationCost( cost )
  {}
  
  long m_installationCost;
};

//*****************************************************************************
//
// TestOneToManyWithProperties
//
// DESCRIPTION:
//
//      Tests one-to-one associations that have properties.
//
//*****************************************************************************

void TestOneToManyWithProperties()
{
  Car car;
  
  Accessory acc1;
  Accessory acc2;
  Accessory acc3;

  // Attach accessory 1 to the car.
  car.m_hasAccessories.Attach( acc1.m_installedIn, *new CarAccessoryProperties( 500 ) );
  
  // Attach accessory 2 to the car.
  acc2.m_installedIn.Attach( car.m_hasAccessories, *new CarAccessoryProperties( 1000 ) );
  
  // Attach accessory 3 to the car.
  acc3.m_installedIn.Attach( car.m_hasAccessories, *new CarAccessoryProperties( 7000 ) );
  
  // The car and the accessories should all be attached to each other.
  if ( !car.m_hasAccessories.IsAttached() )
  {
    printf( "failure\n" );
    return;
  }
  if ( !acc1.m_installedIn.IsAttached() )
  {
    printf( "failure\n" );
    return;
  }
  if ( !acc2.m_installedIn.IsAttached() )
  {
    printf( "failure\n" );
    return;
  }
  if ( !acc3.m_installedIn.IsAttached() )
  {
    printf( "failure\n" );
    return;
  }
  
  // Print out the installation cost for all accessories.
  Iterator *pIterator = car.m_hasAccessories.GetRhsObjects().Iterate(); 
  
  for ( ; !pIterator->AtEnd(); pIterator -> Next() )
  {
    printf( "Cost %d\n", pIterator->GetCurrent().GetProperties().m_installationCost );
  }
  delete pIterator;
  
  printf( "\n" );
  printf( "Accessory 1's properties are %d\n", acc1.m_installedIn.GetProperties().m_installationCost);
  printf( "Accessory 2's properties are %d\n", acc2.m_installedIn.GetProperties().m_installationCost);
  printf( "Accessory 3's properties are %d\n", acc3.m_installedIn.GetProperties().m_installationCost);
}

//*****************************************************************************
// Main
//*****************************************************************************

// Main
void main()
{
  TestOneToManyWithProperties();
}
