//*****************************************************************************
// FILE:  COMPANY.CPP
//
// DESCRIPTION:
//
//      Test program for associations.  Company / board example.
//
// REVISION HISTORY:
//  DATE      WHO   DESCRIPTION
//  1l/04/95  TJL   Created.
// 
//*****************************************************************************

#include <stdio.h>

#include "assn.h"

// Forward
class Company;
class Board;
class CompanyBoardProperties;

//*****************************************************************************
//
// class Company
//
//*****************************************************************************

class Company
{
  public:
  
  Company() : m_board( *this )
  {
  }
  
  ASToOneProp< Company, Board, CompanyBoardProperties > m_board;
};

//*****************************************************************************
//
// class Board 
//
//*****************************************************************************

class Board
{
  public:
  
  Board() : m_company( *this )
  {}
  
  ASToOneProp<Board, Company, CompanyBoardProperties> m_company;
};

//*****************************************************************************
//
// class CompanyBoardProperties
//
//*****************************************************************************

class CompanyBoardProperties
{
  public:
  
  CompanyBoardProperties( const long duration )
    : m_duration( duration )
  {}
  
  long m_duration;
};

//*****************************************************************************
//
// TestOneToOneWithProperties
//
// DESCRIPTION:
//
//      Tests one-to-one associations that have properties.
//
//*****************************************************************************

void TestOneToOneWithProperties()
{
  Company company;
  Board board;

  // Attach accessory 1 to the car.
  company.m_board.Attach( board.m_company, *new CompanyBoardProperties( 5 ) );
  
  // The company and the board should all be attached to each other.
  if ( !company.m_board.IsAttached() )
  {
    printf( "failure\n" );
    return;
  }
  if ( !board.m_company.IsAttached() )
  {
    printf( "failure\n" );
    return;
  }

  printf( "\n" );
  printf( "Company properties %d\n", company.m_board.GetProperties().m_duration);
  printf( "Board properties %d\n", board.m_company.GetProperties().m_duration);
}

//*****************************************************************************
// Main
//*****************************************************************************

// Main
void main()
{
  TestOneToOneWithProperties();
}
