//
// CDDJPrintPreview - customized CPreviewView derivative.
// MSW - Stingray Software -http://www.unx.com/~stingray

#include "stdafx.h"
#include "afxpriv.h"
#include "ddjsamp.h"
#include "DDJPPV.h"
#include "ddjvw.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDDJPrintPreview

IMPLEMENT_DYNCREATE(CDDJPrintPreview, CPreviewView)

CDDJPrintPreview::CDDJPrintPreview()
{
    
}

CDDJPrintPreview::~CDDJPrintPreview()
{
}


BEGIN_MESSAGE_MAP(CDDJPrintPreview, CPreviewView)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDDJPrintPreview drawing
//
// This is where any additional customizations should live.
// This example draws notebook paper.


void CDDJPrintPreview::OnDraw(CDC* pDC)
{
	// don't do anything if not fully initialized
	if (m_pPrintView == NULL || m_dcPrint.m_hDC == NULL)
		return;

	CDdjsampView * pMyView; //DDJ NEW
      
	CPoint ViewportOrg = pDC->GetViewportOrg();
	
	CPen rectPen;
	rectPen.CreatePen(PS_SOLID, 2, GetSysColor(COLOR_WINDOWFRAME));
	CPen shadowPen;
	shadowPen.CreatePen(PS_SOLID, 3, GetSysColor(COLOR_BTNSHADOW));

	//New for DDJ
	CPen redPen;
	redPen.CreatePen(PS_SOLID,2,RGB(255,0,0));
	    
	CPen bluePen;
	bluePen.CreatePen(PS_SOLID,1,RGB(0,0,255));
			  
	m_pPreviewInfo->m_bContinuePrinting = TRUE;     // do this once each paint

	for (UINT nPage = 0; nPage < m_nPages; nPage++)
	{
		int nSavedState = m_dcPrint.SaveDC();       // Save pristine state of DC

		// Use paint DC for print preview output
		m_pPreviewDC->SetOutputDC(pDC->GetSafeHdc());

		m_pPreviewInfo->m_nCurPage = m_nCurrentPage + nPage;

		// Only call PrepareDC if within page range, otherwise use default
		// rect to draw page rectangle
		if (m_nCurrentPage + nPage <= m_pPreviewInfo->GetMaxPage())
			m_pPrintView->OnPrepareDC(m_pPreviewDC, m_pPreviewInfo);

		// Set up drawing rect to entire page (in logical coordinates)
		m_pPreviewInfo->m_rectDraw.SetRect(0, 0,
			m_pPreviewDC->GetDeviceCaps(HORZRES),
			m_pPreviewDC->GetDeviceCaps(VERTRES));
		m_pPreviewDC->DPtoLP(&m_pPreviewInfo->m_rectDraw);

		// Draw empty page on screen

		pDC->SaveDC();          // save the output dc state

		CSize* pRatio = &m_pPageInfo[nPage].sizeScaleRatio;
		CRect* pRect = &m_pPageInfo[nPage].rectScreen;

		if (pRatio->cx == 0)
		{   // page position has not been determined
			PositionPage(nPage);    // compute page position
			if (m_nZoomState != ZOOM_OUT)
				ViewportOrg = -GetDeviceScrollPosition();
		}

		pDC->SetMapMode(MM_TEXT);   // Page Rectangle is in screen device coords
		pDC->SetViewportOrg(ViewportOrg);
		pDC->SetWindowOrg(0, 0);

		pDC->SelectStockObject(HOLLOW_BRUSH);
		pDC->SelectObject(&rectPen);
		pDC->Rectangle(pRect);

		pDC->SelectObject(&shadowPen);

		pDC->MoveTo(pRect->right + 1, pRect->top + 3);
		pDC->LineTo(pRect->right + 1, pRect->bottom + 1);
		pDC->MoveTo(pRect->left + 3, pRect->bottom + 1);
		pDC->LineTo(pRect->right + 1, pRect->bottom + 1);

		// erase background to white (most paper is white)
		CRect rectFill = *pRect;
		rectFill.left += 1;
		rectFill.top += 1;
		rectFill.right -= 2;
		rectFill.bottom -= 2;
		::FillRect(pDC->m_hDC, rectFill, (HBRUSH)GetStockObject(WHITE_BRUSH));

		//New for DDJ - Draw our notebook paper look and feel
		pDC->SelectObject(&redPen);
		int nRedX = pRect->left + pRect->Width()/8; //Draw at 1/8th the width
		pDC->MoveTo(nRedX,pRect->top + 2);
		pDC->LineTo(nRedX,pRect->bottom - 2);

		//Start 1/8 way down
		int nBlueCurrentY = pRect->top + pRect->Height()/8;
		int nBlueYIncrement = (pRect->bottom - nBlueCurrentY)/30;
	        pDC->SelectObject(&bluePen);
		
		for (int nBlueCount = 0; nBlueCount < 30; nBlueCount++){
		    pDC->MoveTo(pRect->left + 3,nBlueCurrentY);
		    pDC->LineTo(pRect->right - 2,nBlueCurrentY);
		    nBlueCurrentY += nBlueYIncrement;
		    
		}//end blue line loop.

		//Now let's do some three-hole-punching!
		//Draw one every 1/4 page except for last one.
		//Make it the size of a blue increment which looks good.
		pDC->SelectObject(rectPen);
		pDC->SelectObject(GetStockObject(BLACK_BRUSH));

		//All holes have same left/right, different top/bottom
		CRect rectHole;		
		rectHole.left = pRect->left + pRect->Width()/16;
		rectHole.right = rectHole.left + nBlueYIncrement;
		
		//Hole 1 
		rectHole.top = pRect->Height()/8;
		rectHole.bottom = rectHole.top + nBlueYIncrement;
		pDC->Ellipse(rectHole);

		//Hole 2
		rectHole.top = pRect->Height()/2;
		rectHole.bottom = rectHole.top + nBlueYIncrement;
		pDC->Ellipse(rectHole);

		//Hole 3
		rectHole.top = pRect->Height()/8 * 7;
		rectHole.bottom = rectHole.top + nBlueYIncrement;
		pDC->Ellipse(rectHole);
		
		
		pDC->RestoreDC(-1);     // restore to synchronized state

		if (!m_pPreviewInfo->m_bContinuePrinting ||
				m_nCurrentPage + nPage > m_pPreviewInfo->GetMaxPage())
		{
			m_pPreviewDC->ReleaseOutputDC();
			m_dcPrint.RestoreDC(nSavedState);   // restore to untouched state

			// if the first page is not displayable, back up one page
			// but never go below 1
			if (nPage == 0 && m_nCurrentPage > 1)
				SetCurrentPage(m_nCurrentPage - 1, TRUE);
			break;
		}

		// Display page number
		OnDisplayPageNumber(m_nCurrentPage, nPage + 1);

		// Set scale ratio for this page
		m_pPreviewDC->SetScaleRatio(pRatio->cx, pRatio->cy);

		CSize PrintOffset;
		VERIFY(m_pPreviewDC->Escape(GETPRINTINGOFFSET, 0, NULL, (LPVOID)&PrintOffset));
		m_pPreviewDC->PrinterDPtoScreenDP((LPPOINT)&PrintOffset);
		PrintOffset += (CSize)pRect->TopLeft();
		PrintOffset += CSize(1, 1);
		PrintOffset += (CSize)ViewportOrg;  // For Scrolling

		m_pPreviewDC->SetTopLeftOffset(PrintOffset);

		m_pPreviewDC->ClipToPage();

		ASSERT(m_pPrintView->IsKindOf(RUNTIME_CLASS(CDdjsampView)));

		pMyView = (CDdjsampView *)m_pPrintView;

		pMyView->OnPrint(m_pPreviewDC,m_pPreviewInfo);
		//m_pPrintView->OnPrint(m_pPreviewDC, m_pPreviewInfo);

		m_pPreviewDC->ReleaseOutputDC();

		m_dcPrint.RestoreDC(nSavedState);   // restore to untouched state

	}

	rectPen.DeleteObject();
	shadowPen.DeleteObject();
	//Nuke our pens..
	bluePen.DeleteObject();
	redPen.DeleteObject();
}

/////////////////////////////////////////////////////////////////////////////
// CDDJPrintPreview diagnostics

#ifdef _DEBUG
void CDDJPrintPreview::AssertValid() const
{
	CView::AssertValid();
}

void CDDJPrintPreview::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDDJPrintPreview message handlers




