//*****************************************************************************
//
// FILE:  LIST.H
//
// DESCRIPTION:
//
//      Defines a very minimal list class that implements the ASListAdapter
//      interface.  Provides enough functionality to let us limp around
//      enough to demonstrate that the design works.
//
// REVISION HISTORY:
//  DATE      WHO   DESCRIPTION
//  11/04/95  TJL   Created.
// 
//*****************************************************************************

#define LIST_H 

#ifndef AS_ASSNLIST_H
#include "assnlist.h"
#endif

//*****************************************************************************
// 
// List template
//
//*****************************************************************************

template < class itemClass, class whatToFindClass >
class IteratorTempl;

template <class itemClass, class whatToFindClass>
class ListTempl : public ASListAdapter< itemClass, whatToFindClass > 
{
  public:
  
    ListTempl()
    {
      m_iItems = 0;
    }
    ~ListTempl()
    {
    }
    void AddItem(itemClass &rItem, int order = 0)
    {
      m_pItems[m_iItems] = &rItem;
      ++m_iItems;
    }
    void RemoveItem(const whatToFindClass&rWhatToFind)
    {
      for (int i = 0; i < m_iItems; i++)
      {
        if ( *m_pItems[i] == rWhatToFind )
        {
          for ( int j = i; j < (m_iItems-1); j++ )
          {
            m_pItems[j] = m_pItems[j+1];
          }
          --m_iItems;
          return;
        }
      }
    }
    void RemoveAll()
    {
      m_iItems = 0;
    }
    itemClass *FindItem(const whatToFindClass &rWhatToFind)
    {
      for (int i = 0; i < m_iItems; i++)
      {
        if ( *m_pItems[i] == rWhatToFind )
        {
          return m_pItems[i];
        }
      }
      return NULL;
    }
    ASIteratorAdapter<itemClass> *Iterate() const
    {
      ListTempl<itemClass, whatToFindClass> * pThis = (ListTempl<itemClass, whatToFindClass> *)this;
      return new IteratorTempl<itemClass, whatToFindClass>( *pThis );
    }
    int GetNumItems() const
    {
      return m_iItems;
    }
    itemClass &GetItem( int iPosition )
    {
      return *m_pItems[iPosition];
    }
    
  private:
  
    itemClass *m_pItems [100]; 
    int m_iItems;
};

//*****************************************************************************
// 
// Iterator template
//
//*****************************************************************************

template <class itemClass, class whatToFindClass>
class IteratorTempl : public ASIteratorAdapter< itemClass >
{
  public:
    
    IteratorTempl( ListTempl<itemClass, whatToFindClass> &rList )
    {
      m_iPosition = 0;
      m_pList = &rList;
    }
    ~IteratorTempl()
    {
    }
    int AtEnd()
    {
      return m_iPosition >= m_pList -> GetNumItems();
    }
    void Next()
    {
      ++m_iPosition;
    }
    itemClass &GetCurrent()
    {
      return m_pList->GetItem(m_iPosition);
    }
    
  private:
  
    int m_iPosition;
    ListTempl<itemClass,whatToFindClass> *m_pList;
};
